@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
 #
 # Podcastparser: A simple, fast and efficient podcast parser
-# Copyright (c) 2012, 2013, 2014, Thomas Perl <m@thp.io>
+# Copyright (c) 2012, 2013, 2014, 2018, 2020 Thomas Perl <m@thp.io>
+# Copyright (c) 2016, 2017, 2018, 2019, 2020 Eric Le Lay <elelay@macports.org>
+# Copyright (c) 2020 E.S. Rosenberg <es.rosenberg+openu@gmail.com>
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -20,7 +22,7 @@
 
 # Will be parsed by setup.py to determine package metadata
 __author__ = 'Thomas Perl <m@thp.io>'
-__version__ = '0.6.2'
+__version__ = '0.6.5'
 __website__ = 'http://gpodder.org/podcastparser/'
 __license__ = 'ISC License'
 
@@ -70,9 +72,10 @@ logger = logging.getLogger(__name__)
 class Target(object):
     WANT_TEXT = False
 
-    def __init__(self, key=None, filter_func=lambda x: x.strip()):
+    def __init__(self, key=None, filter_func=lambda x: x.strip(), overwrite=True):
         self.key = key
         self.filter_func = filter_func
+        self.overwrite = overwrite
 
     def start(self, handler, attrs):
         pass
@@ -129,6 +132,8 @@ class EpisodeAttr(Target):
     WANT_TEXT = True
 
     def end(self, handler, text):
+        if not self.overwrite and handler.get_episode_attr(self.key):
+            return
         handler.set_episode_attr(self.key, self.filter_func(text))
 
 
@@ -174,7 +179,7 @@ class Enclosure(Target):
         if url is None:
             return
 
-        url = parse_url(urlparse.urljoin(handler.base, url))
+        url = parse_url(urlparse.urljoin(handler.base, url.lstrip()))
         file_size = parse_length(attrs.get(self.file_size_attribute))
         mime_type = parse_type(attrs.get('type'))
 
@@ -232,20 +237,42 @@ class AtomContent(Target):
             handler.set_episode_attr('description', squash_whitespace(text))
 
 
+class RSSItemDescription(Target):
+    """
+    RSS 2.0 almost encourages to put html content in item/description
+    but content:encoded is the better source of html content and itunes:summary
+    is known to contain the short textual description of the item.
+    So use a heuristic to attribute text to either description or description_html,
+    without overriding existing values.
+    """
+    WANT_TEXT = True
+
+    def __init__(self):
+        self._want_content = False
+
+    def end(self, handler, text):
+        if is_html(text):
+            if not handler.get_episode_attr('description_html'):
+                handler.set_episode_attr('description_html', text.strip())
+        elif not handler.get_episode_attr('description'):
+            # don't overwrite itunes:summary?
+            handler.set_episode_attr('description', squash_whitespace(text))
+
+
 class PodloveChapters(Target):
     SUPPORTED_VERSIONS = ('1.1', '1.2')
 
     def start(self, handler, attrs):
         version = attrs.get('version', '1.1')
         if version not in PodloveChapters.SUPPORTED_VERSIONS:
-            logger.warn('Possible incompatible chapters version: %s', version)
+            logger.warning('Possible incompatible chapters version: %s', version)
 
 
 class PodloveChapter(Target):
     def start(self, handler, attrs):
         # Both the start and title attributes are mandatory
         if attrs.get('start') is None or attrs.get('title') is None:
-            logger.warn('Invalid chapter (missing start and/or and title)')
+            logger.warning('Invalid chapter (missing start and/or and title)')
             return
 
         chapter = {
@@ -358,7 +385,7 @@ class Namespace():
             namespace_uri = self.lookup(namespace)
             if namespace_uri is None:
                 # Use of "itunes:duration" without xmlns:itunes="..."
-                logger.warn('No namespace defined for "%s:%s"', namespace,
+                logger.warning('No namespace defined for "%s:%s"', namespace,
                             name)
                 return '%s:%s' % (namespace, name)
 
@@ -366,7 +393,7 @@ class Namespace():
             prefix = self.NAMESPACES.get(namespace_uri)
             if prefix is None and namespace:
                 # Proper use of namespaces, but unknown namespace
-                # logger.warn('Unknown namespace: %s', namespace_uri)
+                # logger.warning('Unknown namespace: %s', namespace_uri)
                 # We prefix the tag name here to make sure that it does not
                 # match any other tag below if we can't recognize the namespace
                 name = '!%s:%s' % (namespace, name)
@@ -463,7 +490,7 @@ def parse_time(value):
         try:
             seconds = int(value)
         except ValueError:
-            logger.warn('Could not parse time value: "%s"', value)
+            logger.warning('Could not parse time value: "%s"', value)
             return 0
 
     return (int(hours) * 60 + int(minutes)) * 60 + int(seconds)
@@ -551,7 +578,7 @@ def parse_pubdate(text):
             pubtimeseconds = int(mktime_tz(parsed))
             return pubtimeseconds
         except(OverflowError,ValueError):
-            logger.warn('bad pubdate %s is before epoch or after end of time (2038)',parsed)
+            logger.warning('bad pubdate %s is before epoch or after end of time (2038)',parsed)
             return 0
         
     try:
@@ -593,7 +620,7 @@ MAPPING = {
     'rss/channel/item/guid': EpisodeGuid('guid'),
     'rss/channel/item/title': EpisodeAttr('title', squash_whitespace),
     'rss/channel/item/link': EpisodeAttrRelativeLink('link'),
-    'rss/channel/item/description': EpisodeAttr('description', squash_whitespace),
+    'rss/channel/item/description': RSSItemDescription(),
     'rss/channel/item/itunes:summary': EpisodeAttr('description', squash_whitespace),
     'rss/channel/item/media:description': EpisodeAttr('description', squash_whitespace),
     'rss/channel/item/itunes:subtitle': EpisodeAttr('subtitle', squash_whitespace),
@@ -601,6 +628,7 @@ MAPPING = {
     'rss/channel/item/itunes:duration': EpisodeAttr('total_time', parse_time),
     'rss/channel/item/pubDate': EpisodeAttr('published', parse_pubdate),
     'rss/channel/item/atom:link': AtomLink(),
+    'rss/channel/item/itunes:image': EpisodeAttrFromHref('episode_art_url'),
 
     'rss/channel/item/media:content': Enclosure('fileSize'),
     'rss/channel/item/enclosure': Enclosure('length'),
@@ -611,7 +639,7 @@ MAPPING = {
     'atom:feed': PodcastItem(),
     'atom:feed/atom:title': PodcastAttr('title', squash_whitespace),
     'atom:feed/atom:subtitle': PodcastAttr('description', squash_whitespace),
-    'atom:feed/atom:icon': PodcastAttr('cover_url'),
+    'atom:feed/atom:icon': PodcastAttrRelativeLink('cover_url'),
     'atom:feed/atom:link': PodcastAtomLink(),
     'atom:feed/atom:entry': EpisodeItem(),
     'atom:feed/atom:entry/atom:id': EpisodeAttr('guid'),
@@ -620,7 +648,7 @@ MAPPING = {
     'atom:feed/atom:entry/atom:content': AtomContent(),
     'atom:feed/atom:entry/content:encoded': EpisodeAttr('description_html'),
     'atom:feed/atom:entry/atom:published': EpisodeAttr('published', parse_pubdate),
-    'atom:feed/atom:entry/atom:updated': EpisodeAttr('published', parse_pubdate),
+    'atom:feed/atom:entry/atom:updated': EpisodeAttr('published', parse_pubdate, overwrite=False),
     'atom:feed/atom:entry/media:group/media:description': EpisodeAttr('description', squash_whitespace),
     'atom:feed/atom:entry/psc:chapters': PodloveChapters(),
     'atom:feed/atom:entry/psc:chapters/psc:chapter': PodloveChapter(),
@@ -688,12 +716,12 @@ class PodcastHandler(sax.handler.ContentHandler):
             del entry['chapters']
 
         # Ensures `description` does not contain HTML
-        if 'description' in entry and is_html(entry['description']):
+        if is_html(entry['description']):
             if 'description_html' not in entry:
                 entry['description_html'] = entry['description']
             entry['description'] = ''
 
-        # Sets `description` to stripped `description_html` when absent
+        # Sets `description` to stripped `description_html` when empty
         if 'description_html' in entry and not entry['description']:
             entry['description'] = remove_html_tags(entry['description_html'])
 
@@ -850,12 +878,17 @@ def normalize_feed_url(url):
     # urlunsplit might return "a slighty different, but equivalent URL"
     return urlparse.urlunsplit((scheme, netloc, path, query, fragment))
 
+HTML_TEST = re.compile('<[a-z][a-z0-9]*(?:\s.*?>|\/?>)', re.IGNORECASE | re.DOTALL)
 def is_html(text):
+    """Heuristically tell if text is HTML
+
+    By looking for an open tag (more or less:)
+    >>> is_html('<h1>HELLO</h1>')
+    True
+    >>> is_html('a < b < c')
+    False
     """
-    Tests whether the given string contains HTML encoded data
-    """
-    html_test = re.compile(r'<[a-z][\s\S]*>', re.IGNORECASE)
-    return bool(html_test.search(text))
+    return bool(HTML_TEST.search(text))
 
 def remove_html_tags(html):
     """
